@@ -18,7 +18,6 @@
 
 #include <glib/gi18n.h>
 #include <stdlib.h>
-#include <clutter-gtk/clutter-gtk.h>
 
 #include <libbiji/libbiji.h>
 
@@ -57,11 +56,14 @@ bijiben_new_window_internal (GApplication *app,
 
   if (note != NULL)
     {
-      bjb_window_base_set_frame(BJB_WINDOW_BASE(win),
-                                CLUTTER_ACTOR(bjb_note_view_new(GTK_WIDGET(win), note)));
-
-      gtk_window_set_title(win,biji_note_obj_get_title (note));
+      bjb_window_base_switch_to_note (BJB_WINDOW_BASE (win), note);
     }
+
+  else
+  {
+    bjb_window_base_switch_to (BJB_WINDOW_BASE (win), BJB_MAIN_VIEW);
+    gtk_widget_show_all (GTK_WIDGET (win)); // FIXME not here
+  }
 }
 
 void
@@ -285,10 +287,11 @@ bijiben_startup (GApplication *application)
   G_APPLICATION_CLASS (bijiben_parent_class)->startup (application);
 
   if (gtk_clutter_init (NULL, NULL) != CLUTTER_INIT_SUCCESS)
-    {
-      g_warning ("Unable to initialize Clutter");
-      return;
-    }
+  {
+    g_warning ("Unable to initialize Clutter");
+    return;
+  }
+
 
   bjb_app_menu_set(application);
 
@@ -355,9 +358,6 @@ bijiben_class_init (BijibenClass *klass)
 Bijiben *
 bijiben_new (void)
 {
-  #if !GLIB_CHECK_VERSION(2,35,0) /* return true if glib < 2.35.0 */
-    g_type_init ();
-  #endif
   return g_object_new (BIJIBEN_TYPE_APPLICATION,
                        "application-id", "org.gnome.bijiben",
                        "flags", G_APPLICATION_HANDLES_OPEN,
