@@ -1,6 +1,6 @@
 /*
  * bjb-controller.c
- * Copyright (C) Pierre-Yves Luyten 2012 <py@luyten.fr>
+ * Copyright (C) Pierre-Yves Luyten 2012, 2013 <py@luyten.fr>
  * 
  * bijiben is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -58,10 +58,10 @@ enum {
 
 static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
 
-/* The Controller wants to inform the toolbar when search starts.
- * But other might be interested to know. */
+/* Currently used by toolbars */
 enum {
   SEARCH_CHANGED,
+  DISPLAY_NOTES_CHANGED, // either search or book change
   BJB_CONTROLLER_SIGNALS
 };
 
@@ -86,13 +86,13 @@ bjb_controller_init (BjbController *self)
   priv = self->priv ;
 
   /* Create the columns */
-  store = gtk_list_store_new (NUMBER_COLUMNS,
+  store = gtk_list_store_new (GD_MAIN_COLUMN_LAST,
                               G_TYPE_STRING,   // urn
                               G_TYPE_STRING,   // uri
                               G_TYPE_STRING,   // name
                               G_TYPE_STRING,   // author
                               GDK_TYPE_PIXBUF,   // icon then note
-                              G_TYPE_INT64,    // mtime
+                              G_TYPE_LONG, // mtime
                               G_TYPE_BOOLEAN);   // state
 
   priv->model = GTK_TREE_MODEL(store) ;
@@ -218,19 +218,20 @@ bjb_controller_add_note (BjbController *self,
      * currently use the same model */
     path = biji_note_obj_get_path (note);
 
-    gtk_list_store_set (store, 
-                        &iter,
-                        COL_URN,    path,
-                        COL_URI,    path,
-                        COL_NAME,   biji_note_obj_get_title(note),
-                        COL_AUTHOR,   NULL,
-                        COL_IMAGE,  pix,
-                        COL_MTIME,  biji_note_obj_get_last_change_date_sec(note),
-                        COL_SELECTED, FALSE,
-                        -1);
+    gtk_list_store_set (store, &iter,
+         GD_MAIN_COLUMN_ID, path,
+         GD_MAIN_COLUMN_URI, path,
+         GD_MAIN_COLUMN_PRIMARY_TEXT, biji_note_obj_get_title (note),
+         GD_MAIN_COLUMN_SECONDARY_TEXT, NULL,
+         GD_MAIN_COLUMN_ICON, pix,
+         GD_MAIN_COLUMN_MTIME, biji_note_obj_get_last_change_date_sec (note),
+         GD_MAIN_COLUMN_SELECTED, FALSE,
+         -1);
 
     g_free (path);
   }
+
+
 }
 
 /* If the user searches for notes, is the note searched? */
@@ -326,8 +327,10 @@ sort_notes( BjbController *self)
 static void
 sort_and_update (BjbController *self)
 {
-  sort_notes (self) ;
+  sort_notes (self);
   bjb_controller_update_view (self);
+
+  g_signal_emit (G_OBJECT (self), bjb_controller_signals[DISPLAY_NOTES_CHANGED],0);
 }
 
 static void
@@ -341,22 +344,23 @@ update_controller_callback (GObject *source_object,
   result = biji_get_notes_with_strings_or_collection_finish (source_object, res, self->priv->book);
   self->priv->notes_to_show = result;
   sort_and_update (self);
-}
+}          
 
 void
-bjb_controller_apply_needle ( BjbController *self )
+bjb_controller_apply_needle (BjbController *self)
 {
-  gchar *needle ;
+  BjbControllerPrivate *priv = self->priv;
+  gchar *needle;
 
-  if (self->priv->notes_to_show)
-    g_clear_pointer (&self->priv->notes_to_show, g_list_free);
+  if (priv->notes_to_show)
+    g_clear_pointer (&priv->notes_to_show, g_list_free);
   
-  needle = self->priv->needle;
+  needle = priv->needle;
 
   /* Show all notes */
   if (needle == NULL || g_strcmp0 (needle,"") == 0)
   {
-    self->priv->notes_to_show = biji_note_book_get_notes (self->priv->book);
+    priv->notes_to_show = biji_note_book_get_notes (self->priv->book);
     sort_and_update (self);
     return;
   }
@@ -451,6 +455,7 @@ on_book_changed (BijiNoteBook           *book,
     case BIJI_BOOK_NOTE_ADDED:
         bjb_controller_add_note_if_needed (self, note, TRUE);
         priv->notes_to_show = g_list_prepend (priv->notes_to_show, note);
+        g_signal_emit (G_OBJECT (self), bjb_controller_signals[DISPLAY_NOTES_CHANGED],0);
       break;
 
     /* If the note is *amended*, then per definition we prepend.
@@ -468,12 +473,15 @@ on_book_changed (BijiNoteBook           *book,
       if (gd_main_view_get_view_type (priv->cur) == GD_MAIN_VIEW_ICON
           && bjb_controller_get_iter_at_note (self, note, &p_iter))
         gtk_list_store_set (GTK_LIST_STORE (priv->model), p_iter,
-                            COL_IMAGE, biji_note_obj_get_icon (note), -1);
+                            GD_MAIN_COLUMN_ICON, biji_note_obj_get_icon (note), -1);
       break;
 
     case BIJI_BOOK_NOTE_TRASHED:
       if (bjb_controller_get_iter_at_note (self, note, &p_iter))
         gtk_list_store_remove (GTK_LIST_STORE (priv->model), p_iter);
+
+      priv->notes_to_show = g_list_remove (priv->notes_to_show, note);
+      g_signal_emit (G_OBJECT (self), bjb_controller_signals[DISPLAY_NOTES_CHANGED],0);
       break;
 
     default:
@@ -534,6 +542,16 @@ bjb_controller_class_init (BjbControllerClass *klass)
                                                   G_TYPE_NONE,
                                                   0);
 
+  bjb_controller_signals[DISPLAY_NOTES_CHANGED] = g_signal_new ( "display-notes-changed" ,
+                                                  G_OBJECT_CLASS_TYPE (klass),
+                                                  G_SIGNAL_RUN_LAST,
+                                                  0, 
+                                                  NULL, 
+                                                  NULL,
+                                                  g_cclosure_marshal_VOID__BOOLEAN,
+                                                  G_TYPE_NONE,
+                                                  0);
+
   properties[PROP_BOOK] = g_param_spec_object ("book",
                                                "Book",
                                                "The BijiNoteBook",
@@ -621,4 +639,11 @@ bjb_controller_set_main_view (BjbController *self, GdMainView *current)
   bjb_controller_update_view (self);
 }
 
+gboolean
+bjb_controller_shows_notes (BjbController *self)
+{
+  if (self->priv->notes_to_show)
+    return TRUE;
 
+  return FALSE;
+}
