@@ -84,14 +84,6 @@ bjb_main_view_init (BjbMainView *object)
 static void
 bjb_main_view_finalize (GObject *object)
 {
-  BjbMainView     *self = BJB_MAIN_VIEW(object) ;
-  BjbMainViewPriv *priv = self->priv;
-
-  /* Widgets, actors */
-  g_clear_object (&priv->main_toolbar);
-  g_clear_object (&priv->select_bar);
-  g_clear_object (&priv->search_bar);
-
   G_OBJECT_CLASS (bjb_main_view_parent_class)->finalize (object);
 }
 
@@ -401,7 +393,7 @@ on_item_activated(GdMainView        * gd,
   /* Get Note Path */
   model = gd_main_view_get_model (gd);
   gtk_tree_model_get_iter (model, &iter, (GtkTreePath*) path);
-  gtk_tree_model_get (model, &iter,COL_URI, &note_path,-1);
+  gtk_tree_model_get (model, &iter, GD_MAIN_COLUMN_URI, &note_path,-1);
 
   /* Switch to that note */
   book = bjb_window_base_get_book (view->priv->window); 
