@@ -74,21 +74,39 @@ struct _BjbNoteViewPrivate {
   gulong    color;
 };
 
+static void
+bjb_note_view_disconnect (BjbNoteViewPrivate *priv)
+{
+  if (priv->renamed != 0)
+    g_signal_handler_disconnect (priv->note, priv->renamed);
+
+  if (priv->destroy != 0)
+    g_signal_handler_disconnect (priv->window, priv->destroy);
+
+  if (priv->deleted != 0)
+    g_signal_handler_disconnect (priv->note, priv->deleted);
+
+  if (priv->color != 0)
+    g_signal_handler_disconnect (priv->note, priv->color);
+
+  priv->renamed =0;
+  priv->destroy = 0,
+  priv->deleted = 0;
+  priv->color =0;
+}
+
 static void
 bjb_note_view_finalize(GObject *object)
 {
   BjbNoteView *self = BJB_NOTE_VIEW (object) ;
   BjbNoteViewPrivate *priv = self->priv;
 
-  g_signal_handler_disconnect (priv->note, priv->renamed);
-  g_signal_handler_disconnect (priv->window, priv->destroy);
-  g_signal_handler_disconnect (priv->note, priv->deleted);
-  g_signal_handler_disconnect (priv->note, priv->color);
+  bjb_note_view_disconnect (priv);
 
+  g_clear_object (&priv->view);
   g_clear_object (&priv->accel);
   g_clear_object (&priv->edit_bar);
   clutter_color_free (priv->last_date_bckgrd_clr);
-  clutter_actor_destroy (priv->edit_bar_actor);
 
   G_OBJECT_CLASS (bjb_note_view_parent_class)->finalize (object);
 }
@@ -169,6 +187,9 @@ just_switch_to_main_view(BjbNoteView *self)
 {
   GtkWindow     *window;
 
+  /* Avoid stupid crash */
+  bjb_note_view_disconnect (self->priv);
+
   window = GTK_WINDOW(self->priv->window);
   gtk_window_remove_accel_group (window, self->priv->accel);
 
@@ -205,6 +226,7 @@ action_rename_note_callback (GtkWidget *item, gpointer user_data)
     return ;
 
   biji_note_obj_set_title (priv->note,title);
+  biji_note_obj_save_note (priv->note);
 }
 
 static void
@@ -360,6 +382,9 @@ bjb_note_main_toolbar_new (BjbNoteView *self,
   GtkWidget        *color_button;
   GdkRGBA           color;
   BjbSettings      *settings;
+  gboolean          rtl;
+
+  rtl = (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL);
 
   w = gd_main_toolbar_new();
   gd = GD_MAIN_TOOLBAR(w);
@@ -373,7 +398,7 @@ bjb_note_main_toolbar_new (BjbNoteView *self,
 
   /* Go to main view basically means closing note */
   grid = gtk_box_new(GTK_ORIENTATION_HORIZONTAL,2);
-  notes_icon = get_icon("go-previous-symbolic");
+  notes_icon = get_icon(rtl ? "go-previous-rtl-symbolic" : "go-previous-symbolic");
   gtk_box_pack_start(GTK_BOX(grid),notes_icon,TRUE,TRUE,TRUE);
   /* Button to go back to main view */
   notes_label = gtk_label_new (_("Notes"));
