@@ -1,4 +1,5 @@
 
+#include <glib/gi18n.h>
 #include <glib/gprintf.h>
 #include <stdlib.h>
 
@@ -13,7 +14,7 @@
 
 #define BJB_WIDTH 880
 #define BJB_HEIGHT 600
-#define BIJIBEN_MAIN_WIN_TITLE "Bijiben"
+#define BIJIBEN_MAIN_WIN_TITLE N_("Notes")
 
 #define BJB_DEFAULT_FONT "Serif 10"
 
@@ -64,7 +65,6 @@ bjb_window_base_finalize (GObject *object)
   BjbWindowBase *self = BJB_WINDOW_BASE (object);
   BjbWindowBasePriv *priv = self->priv;
 
-  g_clear_object (&priv->view);
   g_clear_object (&priv->controller);
 
   G_OBJECT_CLASS (bjb_window_base_parent_class)->finalize (object);
@@ -95,7 +95,7 @@ bjb_window_base_constructed (GObject *obj)
 
   gtk_window_set_default_size (GTK_WINDOW (self), BJB_WIDTH, BJB_HEIGHT);
   gtk_window_set_position (GTK_WINDOW (self),GTK_WIN_POS_CENTER);
-  gtk_window_set_title (GTK_WINDOW (self), BIJIBEN_MAIN_WIN_TITLE);
+  gtk_window_set_title (GTK_WINDOW (self), _(BIJIBEN_MAIN_WIN_TITLE));
 
   /* Icon for window. TODO - Should be BjbApp */
   icons_path = bijiben_get_bijiben_dir ();
@@ -253,9 +253,12 @@ bjb_window_base_switch_to_note (BjbWindowBase *bwb, BijiNoteObj *note)
 
   destroy_note_if_needed (bwb);
 
+  priv->note = note;
   priv->note_overlay = gtk_overlay_new ();
   gd_stack_add_named (priv->stack, priv->note_overlay, "note-view");
   priv->note_view = bjb_note_view_new (w, priv->note_overlay, note);
+  g_object_add_weak_pointer (G_OBJECT (priv->note_view),
+                             (gpointer *) &priv->note_view);
 
   bjb_window_base_set_note (bwb, priv->note);
   bjb_window_base_switch_to (bwb, BJB_NOTE_VIEW);
