@@ -62,6 +62,7 @@ struct _BijiNoteObjPrivate
 
   /* Signals */
   gulong note_renamed;
+  gulong save;
 };
 
 /* Properties
@@ -108,7 +109,7 @@ biji_note_obj_init (BijiNoteObj *self)
 
   priv->needs_save = FALSE;
   priv->timeout = biji_timeout_new ();
-  g_signal_connect_swapped (priv->timeout, "timeout",
+  priv->save = g_signal_connect_swapped (priv->timeout, "timeout",
                             G_CALLBACK (on_save_timeout), self);
 
   priv->book = NULL ;
@@ -138,7 +139,8 @@ biji_note_obj_finalize (GObject *object)
   BijiNoteObj        *self = BIJI_NOTE_OBJ(object);
   BijiNoteObjPrivate *priv = self->priv;
 
-  g_object_unref (priv->timeout);
+  if (priv->timeout)
+    g_object_unref (priv->timeout);
 
   if (priv->needs_save)
     on_save_timeout (self);
@@ -325,12 +327,15 @@ biji_note_obj_are_same (BijiNoteObj *a, BijiNoteObj* b)
 gboolean
 biji_note_obj_trash (BijiNoteObj *note_to_kill)
 {
+  BijiNoteObjPrivate *priv = note_to_kill->priv;
   GFile *to_trash, *parent, *trash, *backup_file, *icon;
   gchar *note_name, *parent_path, *trash_path, *backup_path, *icon_path;
   GError *error = NULL;
   gboolean result = FALSE;
 
-  biji_timeout_cancel (note_to_kill->priv->timeout);
+  priv->needs_save = FALSE;
+  biji_timeout_cancel (priv->timeout);
+
   to_trash = biji_note_id_get_file (note_to_kill->priv->id);
 
   /* Don't try to backup a file which does not exist */
@@ -651,8 +656,6 @@ note_obj_set_is_template (BijiNoteObj *n,gboolean is_template)
   n->priv->is_template = is_template ;
 }
 
-/* TODO : see if note beeing deleted. set metadata date
- * and last_change date according to a WHAT param */
 void
 biji_note_obj_save_note (BijiNoteObj *self)
 {
@@ -844,8 +847,6 @@ void
 biji_note_obj_set_html_content (BijiNoteObj *note,
                                 gchar *html)
 {
-  // TODO : queue_save with timeout struct
-
   if (html)
   {
     g_free (note->priv->html);
@@ -862,35 +863,40 @@ biji_note_obj_is_opened (BijiNoteObj *note)
 static void
 _biji_note_obj_close (BijiNoteObj *note)
 {
-  note->priv->editor = NULL;
+  BijiNoteObjPrivate *priv = note->priv;
+  priv->editor = NULL;
 
-  /* TODO : check if note is totaly blank
-   * then delete it */
-
-  /* The title might remain unsert if
-   * - new note
-   * - only one row
-   * In such case we want to change title */
- /* if ( ! biji_note_obj_title_survives (note)
-      && note->priv->raw_text
-      && g_strcmp0 (note->priv->raw_text, "") != 0) 
+  /* Delete if note is totaly blank
+   * Actually we just need to remove it from book
+   * since no change could trigger save */
+  if (!priv->raw_text)
   {
-    biji_note_obj_set_title (note, note->priv->raw_text);
-  } */
+    biji_note_book_remove_note (priv->book, note);
+  }
 
-  if (!biji_note_obj_title_survives (note))
+  /* If the note only has one row. put some title */
+  else if (!biji_note_obj_title_survives (note))
   {
     gchar *title = biji_note_book_get_unique_title (biji_note_obj_get_note_book (note),
-                                                    note->priv->raw_text);
+                                                    priv->raw_text);
     biji_note_obj_set_title (note, title);
     g_free (title);
   }
+
+  /* Else the note is not empty & has more than a row.
+   * But the first row might still be empty.*/
+  else if (!biji_note_id_get_title (priv->id) ||
+           g_strcmp0 (biji_note_id_get_title (priv->id),"")==0)
+  {
+    biji_note_obj_set_title (note, priv->raw_text);
+  }
 }
 
 GtkWidget *
 biji_note_obj_open (BijiNoteObj *note)
 {
   note->priv->editor = biji_webkit_editor_new (note);
+  note->priv->editor = biji_webkit_editor_new (note);
 
   g_signal_connect_swapped (note->priv->editor, "destroy",
                             G_CALLBACK (_biji_note_obj_close), note);
